package hu.eltesoft.modelexecution.m2m.logic.tests;


import hu.eltesoft.modelexecution.m2m.logic.ChangeListenerM2MTranslator;
import hu.eltesoft.modelexecution.m2m.logic.SimpleM2MTranslator;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.SignalEvent;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UMLResource;
import org.junit.After;
import org.junit.BeforeClass;
import org.junit.Test;

import com.google.common.base.Stopwatch;

public class MTMPerformanceTest extends M2MTranslatorTestsBase{

	
	protected static final String UML_TEST_100_MODEL_PATH = "resources/test100.uml";
	protected static final String UML_TEST_1000_MODEL_PATH = "resources/test1000.uml";
	protected static final String UML_TEST_10000_MODEL_PATH = "resources/test10000.uml";

	protected ChangeListenerM2MTranslator initChangeListenerTransformation(String path) {
		try {
			return ChangeListenerM2MTranslator.create(configureEngine(path),listener);
		} catch (IncQueryException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	protected SimpleM2MTranslator initBatchTransformation(String path) {
		try {
			return SimpleM2MTranslator.create(configureEngine(path),listener);
		} catch (IncQueryException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	@BeforeClass
	public static void warmup() {
		System.out.println("WARMUP");
		
		Model model = null;
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getPackageRegistry().put(UMLPackage.eNS_URI,
				UMLPackage.eINSTANCE);
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(UMLResource.FILE_EXTENSION, UMLResource.Factory.INSTANCE);
		URI uri = URI.createFileURI(UML_TEST_10000_MODEL_PATH);
		Resource resource = resourceSet.getResource(uri, true);
		List<EObject> contents = resource.getContents();
		for (EObject eobj : contents) {
			if (eobj instanceof Model) {
				model = (Model) eobj;
			}
		}
		
		SimpleM2MTranslator transformation = null;
		try {
			transformation = ChangeListenerM2MTranslator.create(IncQueryEngine.on(model),new BasicTextChangesListener());
		} catch (IncQueryException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		transformation.fullBuild();

	}
	
	
	@Test
	public void testingCLTransformationOnTest100Model() {
		List<Class> addedClasses = new ArrayList<Class>();
		System.out.println("CL_TEST100; FIRST; SECOND; THIRD");
		String timeRow = "TIME;";
		String heapRow = "HEAP;";
		long usedHeapBefore = logMemoryProperties();
		Stopwatch watch = Stopwatch.createStarted();
		
		ChangeListenerM2MTranslator transformation = initChangeListenerTransformation(UML_TEST_100_MODEL_PATH);

		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		SignalEvent e = (SignalEvent) model.getPackagedElement("Event");
		e.setName("ModifiedEvent");
		for(int i = 0; i<10;i++){
			Class c2 = UMLFactory.eINSTANCE.createClass();
			c2.setName(i+"class");
			model.getPackagedElements().add(c2);
			addedClasses.add(c2);
		}
		
		transformation.rebuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		
		watch.reset().start();
		model.getPackagedElements().removeAll(addedClasses);
		
		e.setName("Event");
		
		transformation.rebuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		System.out.println(timeRow);
		System.out.println(heapRow);

	}
	
	@Test
	public void testingCLTransformationOnTest1000Model() {
		List<Class> addedClasses = new ArrayList<Class>();
		System.out.println("CL_TEST1000; FIRST; SECOND; THIRD");
		String timeRow = "TIME;";
		String heapRow = "HEAP;";
		long usedHeapBefore = logMemoryProperties();
		Stopwatch watch = Stopwatch.createStarted();
		
		ChangeListenerM2MTranslator transformation = initChangeListenerTransformation(UML_TEST_1000_MODEL_PATH);
		
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		
		watch.reset().start();
		
		SignalEvent e = (SignalEvent) model.getPackagedElement("Event");
		e.setName("ModifiedEvent");
		
		for(int i = 0; i<100;i++){
			Class c2 = UMLFactory.eINSTANCE.createClass();
			c2.setName(i+"class");
			model.getPackagedElements().add(c2);
			addedClasses.add(c2);
		}
		
		transformation.rebuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		model.getPackagedElements().removeAll(addedClasses);
		
		e.setName("Event");
		
		transformation.rebuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		System.out.println(timeRow);
		System.out.println(heapRow);


	}
	
	
	@Test
	public void testingCLTransformationOnTest10000Model() {
		List<Class> addedClasses = new ArrayList<Class>();
		System.out.println("CL_TEST10000; FIRST; SECOND; THIRD");
		String timeRow = "TIME;";
		String heapRow = "HEAP;";
		long usedHeapBefore = logMemoryProperties();
		Stopwatch watch = Stopwatch.createStarted();
		
		ChangeListenerM2MTranslator transformation = initChangeListenerTransformation(UML_TEST_10000_MODEL_PATH);
		
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		SignalEvent e = (SignalEvent) model.getPackagedElement("Event");
		e.setName("ModifiedEvent");
		
		for(int i = 0; i<1000;i++){
			Class c2 = UMLFactory.eINSTANCE.createClass();
			c2.setName(i+"class");
			model.getPackagedElements().add(c2);
			addedClasses.add(c2);
		}
		
		transformation.rebuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		e.setName("Event");
		model.getPackagedElements().removeAll(addedClasses);
		
		transformation.rebuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		System.out.println(timeRow);
		System.out.println(heapRow);
	}
	
	@Test
	public void testingBatchTransformationOnTest100Model() {
		List<Class> addedClasses = new ArrayList<Class>();
		System.out.println("BT_TEST100; FIRST; SECOND; THIRD");
		String timeRow = "TIME;";
		String heapRow = "HEAP;";
		long usedHeapBefore = logMemoryProperties();
		Stopwatch watch = Stopwatch.createStarted();
		
		SimpleM2MTranslator transformation = initBatchTransformation(UML_TEST_100_MODEL_PATH);

		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		SignalEvent e = (SignalEvent) model.getPackagedElement("Event");
		e.setName("ModifiedEvent");
		
		for(int i = 0; i<10;i++){
			Class c2 = UMLFactory.eINSTANCE.createClass();
			c2.setName(i+"class");
			model.getPackagedElements().add(c2);
			addedClasses.add(c2);
		}
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		e.setName("Event");
		model.getPackagedElements().removeAll(addedClasses);
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		System.out.println(timeRow);
		System.out.println(heapRow);

	}
	
	@Test
	public void testingBatchTransformationOnTest1000Model() {
		List<Class> addedClasses = new ArrayList<Class>();
		System.out.println("BT_TEST1000; FIRST; SECOND; THIRD");
		String timeRow = "TIME;";
		String heapRow = "HEAP;";
		long usedHeapBefore = logMemoryProperties();
		Stopwatch watch = Stopwatch.createStarted();
		
		SimpleM2MTranslator transformation = initBatchTransformation(UML_TEST_1000_MODEL_PATH);
		
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		SignalEvent e = (SignalEvent) model.getPackagedElement("Event");
		e.setName("ModifiedEvent");
		
		for(int i = 0; i<100;i++){
			Class c2 = UMLFactory.eINSTANCE.createClass();
			c2.setName(i+"class");
			model.getPackagedElements().add(c2);
			addedClasses.add(c2);
		}
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		e.setName("Event");
		model.getPackagedElements().removeAll(addedClasses);
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		System.out.println(timeRow);
		System.out.println(heapRow);


	}
	
	
	@Test
	public void testingBatchTransformationOnTest10000Model() {
		List<Class> addedClasses = new ArrayList<Class>();
		System.out.println("BT_TEST10000; FIRST; SECOND; THIRD");
		String timeRow = "TIME;";
		String heapRow = "HEAP;";
		long usedHeapBefore = logMemoryProperties();
		Stopwatch watch = Stopwatch.createStarted();
		
		SimpleM2MTranslator transformation = initBatchTransformation(UML_TEST_10000_MODEL_PATH);
		
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		SignalEvent e = (SignalEvent) model.getPackagedElement("Event");
		e.setName("ModifiedEvent");
		
		for(int i = 0; i<1000;i++){
			Class c2 = UMLFactory.eINSTANCE.createClass();
			c2.setName(i+"class");
			model.getPackagedElements().add(c2);
			addedClasses.add(c2);
		}
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		watch.reset().start();
		
		e.setName("Event");
		model.getPackagedElements().removeAll(addedClasses);
		
		transformation.fullBuild();
		timeRow+=(watch.elapsed(TimeUnit.MILLISECONDS)+";");
		heapRow+=((logMemoryProperties()-usedHeapBefore)+";");
		
		System.out.println(timeRow);
		System.out.println(heapRow);
	}
	
	@After
	public void  cleanup() {
		
		for(int i=0;i<4;i++){
			Runtime.getRuntime().gc();
			
		}	
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * Calls garbage collector 5 times, sleeps 1 second and logs the used, total and free heap sizes in MByte.
	 * 
	 * @param logger
	 * @return The amount of used heap memory in kBytes
	 */
	private static long logMemoryProperties() {
//		for(int i=0;i<4;i++){
//			Runtime.getRuntime().gc();
//		}
//	
//		try {
//			Thread.sleep(1000);
//		} catch (InterruptedException e) {
//			System.out.println("Sleep after GC interrupted");
//		}
		
		long totalHeapKB = Runtime.getRuntime().totalMemory() / 1024;
		long freeHeapKB = Runtime.getRuntime().freeMemory() / 1024;
		long usedHeapKB = totalHeapKB - freeHeapKB;
		//System.out.println(("Used Heap size: " + usedHeapKB / 1024 + " MByte (Total: " + totalHeapKB / 1024 + " MByte, Free: " + freeHeapKB / 1024 + " MByte)"));
		
		return usedHeapKB;
	}
	
}
