package modelgen

class NotationGenerator extends ModelPartGenerator {

	val IdStorage[] idStroages
	val String modelName

	var Layout layout;

	new(IdStorage[] idStroages, String modelName) {
		this.idStroages = idStroages
		this.modelName = modelName
	}

	override getExtenstion() '''.notation'''

	override generate() {
		var numberOfMachines = idStroages.length
		'''
			<?xml version="1.0" encoding="UTF-8"?>
			<xmi:XMI xmi:version="2.0"
			  xmlns:xmi="http://www.omg.org/XMI"
			  xmlns:ecore="http://www.eclipse.org/emf/2002/Ecore"
			  xmlns:notation="http://www.eclipse.org/gmf/runtime/1.0.2/notation"
			  xmlns:style="http://www.eclipse.org/papyrus/infra/viewpoints/policy/style"
			  xmlns:uml="http://www.eclipse.org/uml2/5.0.0/UML">
			
			  «FOR i : 0 ..< numberOfMachines»
			  	«generate(i)»
			  «ENDFOR»
			</xmi:XMI>
		'''
	}

	def generate(int n) {
		val numberOfStates = idStroages.get(n).stateIds.size - 1
		layout = new Layout(numberOfStates)
		'''
			<notation:Diagram xmi:version="2.0"
			  xmlns:xmi="http://www.omg.org/XMI"
			  xmlns:ecore="http://www.eclipse.org/emf/2002/Ecore"
			  xmlns:notation="http://www.eclipse.org/gmf/runtime/1.0.2/notation"
			  xmlns:style="http://www.eclipse.org/papyrus/infra/viewpoints/policy/style"
			  xmlns:uml="http://www.eclipse.org/uml2/5.0.0/UML" xmi:id="«newId»"
			  type="PapyrusUMLStateMachineDiagram" name="StateMachine«n»" measurementUnit="Pixel">
			  <children xmi:type="notation:Shape" xmi:id="«newId»" type="2000">
			    <children xmi:type="notation:DecorationNode" xmi:id="_«newId»" type="2001">
			      <layoutConstraint xmi:type="notation:Bounds" xmi:id="«newId»" width="«layout.regionWidth»" height="20"/>
			    </children>
			    <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="2002">
			      <children xmi:type="notation:Shape" xmi:id="«newId»" type="3000">
			        <eAnnotations xmi:type="ecore:EAnnotation" xmi:id="«newId»" source="RegionAnnotationKey">
			          <details xmi:type="ecore:EStringToStringMapEntry" xmi:id="«newId»" key="RegionZoneKey" value=""/>
			        </eAnnotations>
			        <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="3002">
			          <children xmi:type="notation:Shape" xmi:id="«idStroages.get(n).initShapeId»" type="8000">
			            <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="8001">
			              <layoutConstraint xmi:type="notation:Location" xmi:id="«newId»" x="25" y="3"/>
			            </children>
			            <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="8002">
			              <layoutConstraint xmi:type="notation:Location" xmi:id="«newId»" x="25" y="-10"/>
			            </children>
			            <element xmi:type="uml:Pseudostate" href="«modelName».uml#«idStroages.get(n).initStateId»"/>
			            <layoutConstraint xmi:type="notation:Bounds" xmi:id="«newId»" x="«Layout.gap»" y="«Layout.gap»"/>
			          </children>
			          «FOR i : 0 ..< numberOfStates»
			          	«generateState(n, i)»
			          «ENDFOR»
			          <layoutConstraint xmi:type="notation:Bounds" xmi:id="«newId»"/>
			        </children>
			        <element xmi:type="uml:Region" href="«modelName».uml#«idStroages.get(n).regionId»"/>
			        <layoutConstraint xmi:type="notation:Bounds" xmi:id="«newId»" width="«layout.regionWidth»" height="«layout.
				regionHeight»"/>
			      </children>
			      <layoutConstraint xmi:type="notation:Bounds" xmi:id="«newId»" y="20" width="«layout.regionWidth»" height="«layout.
				regionHeight»"/>
			    </children>
			    <element xmi:type="uml:StateMachine" href="«modelName».uml#«idStroages.get(n).stateMachineId»"/>
			    <layoutConstraint xmi:type="notation:Bounds" xmi:id="«newId»" x="30" y="30" width="«layout.regionWidth»" height="«20 +
				layout.regionHeight»"/>
			  </children>
			  <styles xmi:type="notation:StringValueStyle" xmi:id="«newId»" name="diagram_compatibility_version" stringValue="1.0.0"/>
			  <styles xmi:type="notation:DiagramStyle" xmi:id="«newId»"/>
			  <styles xmi:type="style:PapyrusViewStyle" xmi:id="«newId»">
			    <owner xmi:type="uml:Model" href="«modelName».uml#«IdStorage::modelId»"/>
			  </styles>
			  <element xmi:type="uml:StateMachine" href="«modelName».uml#«idStroages.get(n).stateMachineId»"/>
			  «FOR i : 0 ..< numberOfStates»
			  	«generateTransition(n, i - 1, i)»
			  «ENDFOR»
			  «generateTransition(n, numberOfStates - 1, 0)»
			</notation:Diagram>
		'''
	}

	def generateState(int n, int number) {
		val x = layout.getStateX(number);
		val y = layout.getStateY(number);
		val shapeId = newId
		idStroages.get(n).putShape(number, shapeId)
		'''
			<children xmi:type="notation:Shape" xmi:id="«shapeId»" type="6000">
			  <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="6001"/>
			  <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="6002">
			    <layoutConstraint xmi:type="notation:Bounds" xmi:id="«newId»"/>
			  </children>
			  <element xmi:type="uml:State" href="«modelName».uml#«idStroages.get(n).getState(number)»"/>
			  <layoutConstraint xmi:type="notation:Bounds" xmi:id="«newId»" x="«x»" y="«y»"/>
			</children>
		'''
	}

	def generateTransition(int n, int source, int target) '''
		<edges xmi:type="notation:Connector" xmi:id="«newId»" type="7000"
		  source="«idStroages.get(n).getShape(source)»"
		  target="«idStroages.get(n).getShape(target)»">
		  <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="7001">
		    <layoutConstraint xmi:type="notation:Location" xmi:id="«newId»" y="«layout.getStateY(source)»"/>
		  </children>
		  <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="7002">
		    <layoutConstraint xmi:type="notation:Location" xmi:id="«newId»"/>
		  </children>
		  <children xmi:type="notation:DecorationNode" xmi:id="«newId»" type="7003">
		    <layoutConstraint xmi:type="notation:Location" xmi:id="«newId»" y="«layout.getStateY(target)»"/>
		  </children>
		  <styles xmi:type="notation:FontStyle" xmi:id="«newId»"/>
		  <element xmi:type="uml:Transition" href="«modelName».uml#«idStroages.get(n).getTransitionId(source, target)»"/>
		  <bendpoints xmi:type="notation:RelativeBendpoints" xmi:id="«newId»" points="[-1, 10, 0, -42]$[-10, 43, -9, -9]"/>
		  <targetAnchor xmi:type="notation:IdentityAnchor" xmi:id="«newId»" id="(0.25,0.225)"/>
		</edges>
	'''
}
