package modelgen

class UMLGenerator extends ModelPartGenerator {

	val IdStorage[] idStroages
	val int numberOfStates;

	val eventId = newId

	new(IdStorage[] idStroages, int numberOfStates) {
		this.idStroages = idStroages
		this.numberOfStates = numberOfStates
	}

	override getExtenstion() '''.uml'''

	override generate() {
		var numberOfMachines = idStroages.length
		'''
			<?xml version="1.0" encoding="UTF-8"?>
			<uml:Model xmi:version="20131001"
			  xmlns:xmi="http://www.omg.org/spec/XMI/20131001"
			  xmlns:ecore="http://www.eclipse.org/emf/2002/Ecore"
			  xmlns:uml="http://www.eclipse.org/uml2/5.0.0/UML"
			  xmi:id="«IdStorage::modelId»" name="model">
			  «FOR i : 0 ..< numberOfMachines»
			  	«generate(i)»
			  «ENDFOR»
			  <profileApplication xmi:type="uml:ProfileApplication" xmi:id="«newId»">
			  <eAnnotations xmi:type="ecore:EAnnotation" xmi:id="«newId»" source="http://www.eclipse.org/uml2/2.0.0/UML">
			  <references xmi:type="ecore:EPackage" href="pathmap://PAPYRUS_ACTIONLANGUAGE_PROFILE/ActionLanguage-Profile.profile.uml#_sYse0YZLEeKyw49uR6nx3g"/>
			  </eAnnotations>
			  <appliedProfile xmi:type="uml:Profile" href="pathmap://PAPYRUS_ACTIONLANGUAGE_PROFILE/ActionLanguage-Profile.profile.uml#ActionLanguage"/>
			  </profileApplication>
			</uml:Model>
		'''
	}

	def generate(int n) {
		val signalId = newId
		var operationId = newId
		var methodId = newId
		'''
			<packagedElement xmi:type="uml:SignalEvent" xmi:id="«eventId»" name="Event«n»" signal="«signalId»"/>
			<packagedElement xmi:type="uml:Class" xmi:id="_GWFXsPRIEeSWytuUv5p0RQ"
			  name="Main«n»"
			  classifierBehavior="«idStroages.get(n).stateMachineId»">
			  <ownedBehavior xmi:type="uml:StateMachine" xmi:id="«idStroages.get(n).stateMachineId»" name="StateMachine«n»x«numberOfStates»">
			    <region xmi:type="uml:Region" xmi:id="«idStroages.get(n).regionId»" name="Region">
			      <subvertex xmi:type="uml:Pseudostate" xmi:id="«idStroages.get(n).initStateId»" name="Init"/>
			        «FOR i : 0 ..< numberOfStates»
			        	«generateState(n, i)»
			        «ENDFOR»
			        «FOR i : 0 ..< numberOfStates»
			        	«generateTransition(n, i - 1, i, transitionName(i - 1, i))»
			        «ENDFOR»
			        «generateTransition(n, numberOfStates - 1, 0, transitionName(numberOfStates - 1, 0))»
			    </region>
			  </ownedBehavior>
			  <ownedOperation xmi:type="uml:Operation" xmi:id="«operationId»" name="feed" method="«methodId»"/>
			  <nestedClassifier xmi:type="uml:OpaqueBehavior" xmi:id="«methodId»" name="FeedBehavior" specification="«operationId»">
			    <language>Alf</language>
			    <body>// See action code on State0 entry</body>
			  </nestedClassifier>
			  <nestedClassifier xmi:type="uml:Signal" xmi:id="«signalId»" name="Signal«n»"/>
			  <ownedReception xmi:type="uml:Reception" xmi:id="«newId»" name="Reception" signal="«signalId»"/>
			</packagedElement>
		'''
	}

	def generateState(int n, int number) {
		val id = newId
		idStroages.get(n).putState(number, id)
		val name = "State" + number
		if (number > 0) '''<subvertex xmi:type="uml:State" xmi:id="«id»" name="«name»"/>''' else '''
			<subvertex xmi:type="uml:State" xmi:id="«id»" name="«name»">
			  <entry xmi:type="uml:OpaqueBehavior" xmi:id="«newId»" name="Action">
			    <language>Alf</language>
			    <body>
			        «FOR i : 0 ..< numberOfStates»
			        	this.Reception();
			        «ENDFOR»
			    </body>
			  </entry>
			</subvertex>
		'''
	}

	def generateTransition(int n, int source, int target, CharSequence name) {
		val sourceId = idStroages.get(n).getState(source)
		val targetId = idStroages.get(n).getState(target)
		val id = if(sourceId == idStroages.get(n).initTransitionId) idStroages.get(n).initTransitionId else newId
		idStroages.get(n).putTransitionId(source, target, id)
		'''
			<transition xmi:type="uml:Transition" xmi:id="«id»" name="«name»" source="«sourceId»" target="«targetId»">
			  <trigger xmi:type="uml:Trigger" xmi:id="«newId»" name="«name»Trigger" event="«eventId»"/>
			</transition>
		'''
	}

	def transitionName(int source, int target) '''TFrom«if(source < 0) "Init" else source»To«target»'''
}
