package hu.eltesoft.modelexecution.m2m.logic.viatra.tests;

import static org.junit.Assert.assertEquals;
import hu.eltesoft.modelexecution.m2m.logic.viatra.ChangeListenerBatchTransformation;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Pseudostate;
import org.eclipse.uml2.uml.PseudostateKind;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.UMLFactory;
import org.junit.Test;

public class ChangeListenerM2MTranslatorTests extends IncQueryBasedTestCase {

	private ChangeListenerBatchTransformation transformation;
	protected Resource resource;
	

	protected void initTranslator(String inPath, String outPath) {
		resource = createResource(outPath);
		transformation = new ChangeListenerBatchTransformation();
		transformation.initialize(resource, configureEngine(inPath));
	}

	@Test
	public void testingFullBuildOnSimpleModelRepeteadly() {
		initTranslator(UML_TEST_SIMPLE_MODEL_PATH,UML_TEST_SIMPLE_OUT_PATH);

		for (int i = 0; i < 3; ++i) {
			transformation.execute();
		}

		assertEquals(UML_TEST_SIMPLE_MODEL_EXPECTED_FILES.length, resource.getContents().size());

	}

	@Test
	public void testingFullBuild() {
		initTranslator(UML_TEST_SIMPLE_MODEL_PATH,UML_TEST_SIMPLE_OUT_PATH);

		transformation.execute();

		assertEquals(UML_TEST_SIMPLE_MODEL_EXPECTED_FILES.length, resource.getContents().size());

	}
	
	@Test
	public void testingFullBuildOnTest10000Model() {
		initTranslator(UML_TEST_LARGE_MODEL_PATH, UML_TEST_LARGE_OUT_PATH);


		transformation.execute();

		assertEquals(UML_TEST_LARGE_MODEL_EXPECTED_FILES.length, resource.getContents().size());

	}

	@Test
	public void testingIncrementalModification() {
		initTranslator(UML_TEST_SIMPLE_MODEL_PATH,UML_TEST_SIMPLE_OUT_PATH);
		transformation.execute();

		Class a = (Class) model.getPackagedElement("A");
		StateMachine sm = (StateMachine) a.getOwnedBehavior("SM1");
		Region rg = sm.getRegion("R1");

		Pseudostate init = UMLFactory.eINSTANCE.createPseudostate();
		init.setContainer(rg);
		init.setName("init");
		init.setKind(PseudostateKind.INITIAL_LITERAL);

		State s1 = UMLFactory.eINSTANCE.createState();
		s1.setContainer(rg);
		s1.setName("S2");

		Transition t1 = UMLFactory.eINSTANCE.createTransition();
		t1.setContainer(rg);

		t1.setSource(init);
		t1.setTarget(s1);
		

		init.getOutgoings().add(t1);
		s1.getIncomings().add(t1);
		
		//Add class
		Class c2 = UMLFactory.eINSTANCE.createClass();
		c2.setName("B");
		model.getPackagedElements().add(c2);
		
		transformation.execute();
		assertEquals(3, resource.getContents().size());
		rg.getSubvertices().remove(s1);
		//remove class
		
		model.getPackagedElements().remove(c2);
		
		transformation.execute();
		assertEquals(UML_TEST_SIMPLE_MODEL_EXPECTED_FILES.length, resource.getContents().size());
	}

	@Test
	public void testingIncrementalDeletion() {
		initTranslator(UML_TEST_2015_Q1_MODEL_PATH,UML_TEST_2015_Q1_OUT_PATH);
		transformation.execute();

		Class a = (Class) model.getPackagedElement("Class1");
		StateMachine sm = (StateMachine) a.getOwnedBehavior("StateMachine1");
		Region rg = sm.getRegion("Region1");

		rg.setName("RenamedRegion1");

		transformation.execute();

		assertEquals(UML_TEST_2015_Q1_MODEL_EXPECTED_FILES.length, resource.getContents().size());

	}

	@Test
	public void testingIncrementalDeletionUndo() {
		initTranslator(UML_TEST_2015_Q1_MODEL_PATH,UML_TEST_2015_Q1_OUT_PATH);
		transformation.execute();

		Class a = (Class) model.getPackagedElement("Class1");
		StateMachine sm = (StateMachine) a.getOwnedBehavior("StateMachine1");
		Region rg = sm.getRegion("Region1");

		rg.setName("RenamedRegion1");

		rg.setName("Region1");

		transformation.execute();
		assertEquals(UML_TEST_2015_Q1_MODEL_EXPECTED_FILES.length, resource.getContents().size());
	}

}
