package hu.eltesoft.modelexecution.m2m.logic.viatra

import com.google.common.base.Stopwatch
import hu.eltesoft.modelexecution.m2m.logic.viatra.util.RuleProvider
import java.util.concurrent.TimeUnit
import org.apache.log4j.Logger
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.incquery.runtime.api.GenericPatternGroup
import org.eclipse.incquery.runtime.api.IncQueryEngine
import org.eclipse.viatra.emf.runtime.rules.batch.BatchTransformationStatements
import org.eclipse.viatra.emf.runtime.transformation.batch.BatchTransformation

import static com.google.common.base.Preconditions.*
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.Behavior
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.Event
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.Region
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.Signal
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.Classdef

class MTMBatchTransformation extends MTMTransformation {
	extension Logger logger = Logger.getLogger("cps.xform.m2m.batch.viatra")

	/* Transformation-related extensions */
	extension RuleProvider ruleProvider
	extension BatchTransformation transformation
	extension BatchTransformationStatements statements

	Resource res
	IncQueryEngine engine


	private var initialized = false;

	override initialize(Resource targetResource, IncQueryEngine engine) {
		checkArgument(targetResource != null, "Target resource cannot be null!")
		checkArgument(engine != null, "Engine cannot be null!")

		if (!initialized) {

			this.res = targetResource
			this.engine = engine
			targetResource.contents.clear
			transformation = BatchTransformation::forEngine(engine)
			statements = new BatchTransformationStatements(transformation)

			debug("Preparing queries on engine.")
			var watch = Stopwatch.createStarted
			queryGroup.prepare(engine)
			debug('''Prepared queries on engine («watch.elapsed(TimeUnit.MILLISECONDS)» ms''')

			debug('''Preparing transformation rules.''')
			watch = Stopwatch.createStarted
			ruleProvider = new RuleProvider(engine, res)

			debug('''Prepared transformation rules («watch.elapsed(TimeUnit.MILLISECONDS)» ms)''')
			initialized = true
		}
	}

	override execute() {
		debug('''Executiong transformation''')
		res.contents.clear
		
		behaviorRule.fireAllCurrent
		clsRule.fireAllCurrent
		regionRule.fireAllCurrent
		eventRule.fireAllCurrent
		signalRule.fireAllCurrent
	}

	override dispose() {
		if (transformation != null) {
			transformation.dispose
		}
		transformation = null
		return
	}

	private def getQueryGroup() {
		GenericPatternGroup.of(
			// Insert patterns here
			Behavior.instance(),
			Classdef.instance(),
			Event.instance(),
			Region.instance(),
			Signal.instance()
		);
	}

}