package hu.eltesoft.modelexecution.m2m.logic.viatra.util

import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.BehaviorMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.ClsMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.EventMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.RegionMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.SignalMatcher
import hu.eltesoft.modelexecution.m2m.metamodel.base.ModelRoot
import java.util.Map
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.incquery.runtime.api.IPatternMatch
import org.eclipse.incquery.runtime.api.IncQueryEngine
import org.eclipse.incquery.runtime.api.IncQueryMatcher
import org.eclipse.uml2.uml.Element
import org.eclipse.viatra.emf.runtime.rules.batch.BatchTransformationRule
import org.eclipse.viatra.emf.runtime.rules.batch.BatchTransformationRuleFactory

import static extension hu.eltesoft.modelexecution.m2m.logic.viatra.util.TransformationActions.*

class RuleProvider {
	extension BatchTransformationRuleFactory = new BatchTransformationRuleFactory

	IncQueryEngine engine
	Resource resource

	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> clsRule
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> behaviorRule
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> eventRule
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> regionRule
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> signalRule
	
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> clsIncrRule
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> behaviorIncrRule
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> eventIncrRule
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> regionIncrRule
	BatchTransformationRule<? extends IPatternMatch, ? extends IncQueryMatcher<?>> signalIncrRule

	new(IncQueryEngine engine, Resource res) {
		this.engine = engine
		this.resource = res
	}

	public def getBehaviorRule() {
		if (behaviorRule == null) {
			behaviorRule = createRule(BehaviorMatcher.querySpecification) [ match |
				match.addElements(engine,resource)
			]
		}
		return behaviorRule
	}
	
	

	public def getClsRule() {
		if (clsRule == null) {
			clsRule = createRule(ClsMatcher.querySpecification) [ match |
				match.addElements(engine,resource)
			]
		}
		return clsRule
	}

	public def getRegionRule() {
		if (regionRule == null) {
			regionRule = createRule(RegionMatcher.querySpecification) [ match |
				match.addElements(engine,resource)
			]
		}
		return regionRule
	}

	public def getEventRule() {
		if (eventRule == null) {
			eventRule = createRule(EventMatcher.querySpecification) [ match |
				match.addElements(engine,resource)
			]
		}
		return eventRule
	}

	public def getSignalRule() {
		if (signalRule == null) {
			signalRule = createRule(SignalMatcher.querySpecification) [ match |
				match.addElements(engine,resource)
			]
		}
		return signalRule
	}
	
	//Rules for Change Listening MTM Transformation
	
	
	public def getBehaviorIncrRule(Map<Element, ModelRoot> traceMap) {
		if (behaviorIncrRule == null) {
			behaviorIncrRule = createRule(BehaviorMatcher.querySpecification) [ match |
				traceMap.put(match.behavior,match.addElements(engine,resource))
			]
		}
		return behaviorIncrRule
	}
	
	public def getClsIncrRule(Map<Element, ModelRoot> traceMap) {
		if (clsIncrRule == null) {
			clsIncrRule = createRule(ClsMatcher.querySpecification) [ match |
				traceMap.put(match.cls,match.addElements(engine,resource))
			]
		}
		return clsIncrRule
	}

	public def getRegionIncrRule(Map<Element, ModelRoot> traceMap) {
		if (regionIncrRule == null) {
			regionIncrRule = createRule(RegionMatcher.querySpecification) [ match |
				traceMap.put(match.region,match.addElements(engine,resource))
			]
		}
		return regionIncrRule
	}

	public def getEventIncrRule(Map<Element, ModelRoot> traceMap) {
		if (eventIncrRule == null) {
			eventIncrRule = createRule(EventMatcher.querySpecification) [ match |
				traceMap.put(match.event,match.addElements(engine,resource))
			]
		}
		return eventIncrRule
	}

	public def getSignalIncrRule(Map<Element, ModelRoot> traceMap) {
		if (signalIncrRule == null) {
			signalIncrRule = createRule(SignalMatcher.querySpecification) [ match |
				traceMap.put(match.signal,match.addElements(engine,resource))
			]
		}
		return signalIncrRule
	}

	
}