package hu.eltesoft.modelexecution.m2m.logic.viatra.util

import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.AlfCodeMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.BehaviorMatch
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.ClsMatch
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.ContainerClassOfBehaviorMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.ContainerClassOfRegionMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.EntryMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.EventMatch
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.ExitMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.InitialsMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.MethodMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.OperationMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.ReceptionMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.RegionMatch
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.RegionOfClassMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.SignalEventMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.SignalMatch
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.StateMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.TransitionEffectMatcher
import hu.eltesoft.modelexecution.m2m.logic.viatra.queries.TransitionMatcher
import hu.eltesoft.modelexecution.m2m.metamodel.behavior.BehaviorFactory
import hu.eltesoft.modelexecution.m2m.metamodel.classdef.ClassdefFactory
import hu.eltesoft.modelexecution.m2m.metamodel.event.EventFactory
import hu.eltesoft.modelexecution.m2m.metamodel.region.RegionFactory
import hu.eltesoft.modelexecution.m2m.metamodel.region.RgState
import hu.eltesoft.modelexecution.m2m.metamodel.signal.SignalFactory
import hu.eltesoft.modelexecution.m2t.smap.emf.Reference
import java.util.HashMap
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.incquery.runtime.api.IncQueryEngine
import org.eclipse.uml2.uml.State
import org.eclipse.incquery.runtime.api.IPatternMatch
import hu.eltesoft.modelexecution.m2m.metamodel.signal.SgSignal
import hu.eltesoft.modelexecution.m2m.metamodel.event.EvSignalEvent
import hu.eltesoft.modelexecution.m2m.metamodel.behavior.BhBehavior
import hu.eltesoft.modelexecution.m2m.metamodel.classdef.ClClass
import hu.eltesoft.modelexecution.m2m.metamodel.region.RgRegion

class TransformationActions {
	protected static extension BehaviorFactory BHFACTORY = BehaviorFactory.eINSTANCE;
	protected static extension ClassdefFactory CDFACTORY = ClassdefFactory.eINSTANCE;
	protected static extension RegionFactory RFACTORY = RegionFactory.eINSTANCE;
	protected static extension EventFactory EFACTORY = EventFactory.eINSTANCE;
	protected static extension SignalFactory SFACTORY = SignalFactory.eINSTANCE;

	
	
	def static dispatch addElements(BehaviorMatch match, IncQueryEngine engine, Resource resource) {
		val containerClass = ContainerClassOfBehaviorMatcher.on(engine).getOneArbitraryMatch(match.behavior, null)?.cls
		var temp = AlfCodeMatcher.on(engine).getOneArbitraryMatch(match.behavior, null)?.alfCode
		if (temp == null) {
			temp = "{}"
		}
		val alf = temp;
		val rootBehavior = createBhBehavior => [
			name = match.behavior.name
			alfCode = alf
			containerClass = createBhClass => [
				name = containerClass?.name
			]
		]
		resource.contents.add(rootBehavior)
		rootBehavior
	}

	def static dispatch addElements(ClsMatch match, IncQueryEngine engine, Resource resource) {
		val rootcl = createClClass => [
			name = match.cls.name
		]
		
		RegionOfClassMatcher.on(engine).getAllMatches(match.cls, null).forEach [ rmatch |
			val clRegion = createClRegion => [
				name = rmatch.region.name
			]
			rootcl.setRegion(clRegion)
		]

		OperationMatcher.on(engine).getAllMatches(match.cls, null).forEach [ omatch |

			val method = MethodMatcher.on(engine).getOneArbitraryMatch(null, omatch.operation, null)?.method
			val clOperation = createClOperation => [
				name = omatch.operation.name
				method = method?.name
			]
			rootcl.operations.add(clOperation)
		]

		ReceptionMatcher.on(engine).getAllMatches(match.cls, null, null).forEach [ rmatch |
			val clReception = createClReception => [
				name = rmatch.reception.name
				signal = createClSignal => [
					name = rmatch.signal.name
				]
			]
			rootcl.receptions.add(clReception);
		]
		resource.contents.add(rootcl)
		rootcl
	}

	def static dispatch addElements(RegionMatch match, IncQueryEngine engine, Resource resource) {
		// region init
		val containerClass = ContainerClassOfRegionMatcher.on(engine).getOneArbitraryMatch(match.region, null)?.cls
		val rootRegion = createRgRegion => [
			name = match.regionName
			it.containerClass = createRgClass => [
				it.name = containerClass?.name
			]
		]
		// add states
		val stateMap = new HashMap<State, RgState>();

		StateMatcher.on(engine).getAllMatches(match.region, null).forEach [ rmatch |
			val entry = EntryMatcher.on(engine).getOneArbitraryMatch(null, rmatch.state, null)?.entry
			val exit = ExitMatcher.on(engine).getOneArbitraryMatch(null, rmatch.state, null)?.exit
			val rgState = createRgState => [
				name = rmatch.state.name
				reference = new Reference(rmatch.state)
				if (entry != null) {
					it.entry = createRgBehavior => [
						name = entry.name

					]
				}
				if (exit != null) {
					it.exit = createRgBehavior => [
						name = exit.name

					]
				}
			]

			stateMap.put(rmatch.state, rgState);
			rootRegion.getStates().add(rgState);
		]

		// add initial pseudostate
		InitialsMatcher.on(engine).getAllMatches(match.region, null, null, null).forEach [ rmatch |
			val trEffect = TransitionEffectMatcher.on(engine).getOneArbitraryMatch(match.region, rmatch.initTransition,
				null)?.effect
			val rgInitialPseudostate = createRgInitialPseudostate => [
				it.name = rmatch.initPseudostate.name
				it.reference = new Reference(rmatch.initPseudostate)
				it.initialTransition = createRgTransition => [
					it.reference = new Reference(rmatch.initTransition)
					it.target = stateMap.get(rmatch.firstState)
					if (trEffect != null) {
						it.effect = createRgBehavior => [
							name = trEffect.name
						]
					}
				]
			]
			rootRegion.initialPseudostate = rgInitialPseudostate
		]

		// add transitions
		TransitionMatcher.on(engine).getAllMatches(match.region, null, null, null, null).forEach [ rmatch |
			val trEffect = TransitionEffectMatcher.on(engine).getOneArbitraryMatch(match.region, rmatch.transition,
				null)?.effect
			val rgSourceState = stateMap.get(rmatch.source);
			rgSourceState.getTransitions().add(createRgTransition => [
				it.reference = new Reference(rmatch.transition)
				it.target = stateMap.get(rmatch.target)
				if (trEffect != null) {
					it.effect = createRgBehavior => [
						name = trEffect.name
					]
				}
				if (rmatch.event.name != null) {
					it.event = createRgEvent => [
						name = rmatch.event.name
					]
				}
			])
		]
		resource.contents.add(rootRegion)
		rootRegion
	}

	def static dispatch addElements(EventMatch match, IncQueryEngine engine, Resource resource) {
		val signal = SignalEventMatcher.on(engine).getOneArbitraryMatch(match.event, null)?.signal
		val rootEvent = createEvSignalEvent => [
			name = match.event.name
			if (signal != null) {
				it.signal = createEvSignal => [
					name = signal.name
				]
			}
		]
		resource.contents.add(rootEvent)
		rootEvent
	}

	def static dispatch addElements(SignalMatch match, IncQueryEngine engine, Resource resource) {
		val rootSignal = createSgSignal => [
			name = match.signalName
		]
		resource.contents.add(rootSignal)
		rootSignal
	}
	
	def static dispatch addElements(IPatternMatch match, IncQueryEngine engine, Resource resource) {
	
	}
	
	// REMOVE ELEMENTS
	
	def static dispatch removeElement(BhBehavior behavior) {
		val res = behavior.eResource
		behavior.alfCode = ""
		behavior.containerClass = null;
		res.contents.remove(behavior)
	}

	def static dispatch removeElement(ClClass clClass) {
		val res = clClass.eResource
		clClass.region = null;
		clClass.operations.clear
		clClass.receptions.clear
		res.contents.remove(clClass)
	}

	def static dispatch removeElement(RgRegion rgRegion) {
		val res = rgRegion.eResource
		rgRegion.containerClass = null;
		rgRegion.states.forEach[state | 
			state.entry = null;
			state.exit =null;
			state.reference = null;
			state.transitions.forEach[tr |
				tr.reference = null
				tr.target = null
				tr.effect = null
			]
			state.transitions.clear
		]
		rgRegion.states.clear
		rgRegion.initialPseudostate.initialTransition.reference = null
		rgRegion.initialPseudostate.initialTransition.target = null
		rgRegion.initialPseudostate.initialTransition.effect = null
		res.contents.remove(rgRegion)
	}

	def static dispatch removeElement(EvSignalEvent event) {
		val res = event.eResource
		event.signal = null;
		res.contents.remove(event);
	}

	def static dispatch removeElement(SgSignal signal) {
		val res = signal.eResource
		res.contents.remove(signal);
	}
	

}